@@ -1,8 +1,10 @@
 use std::fmt::{self, Display, Formatter};
 
-use darling::ast::{Data, Fields};
-use darling::util::{Ignored, SpannedValue};
-use darling::{FromDeriveInput, FromField, FromMeta, FromVariant};
+use darling::{
+    ast::{Data, Fields},
+    util::{Ignored, SpannedValue},
+    FromDeriveInput, FromField, FromMeta, FromVariant,
+};
 use inflector::Inflector;
 use syn::{
     Attribute, Generics, Ident, Lit, LitBool, LitStr, Meta, NestedMeta, Path, Type, Visibility,
@@ -154,6 +156,8 @@ pub struct SimpleObjectField {
     pub visible: Option<Visible>,
     #[darling(default, multiple)]
     pub derived: Vec<DerivedField>,
+    #[darling(default)]
+    pub process_with: Option<String>,
     // for InputObject
     #[darling(default)]
     pub default: Option<DefaultValue>,
@@ -184,6 +188,8 @@ pub struct SimpleObject {
     #[darling(default)]
     pub name: Option<String>,
     #[darling(default)]
+    pub name_type: bool,
+    #[darling(default)]
     pub rename_fields: Option<RenameRule>,
     #[darling(default)]
     pub rename_args: Option<RenameRule>,
@@ -212,6 +218,8 @@ pub struct Argument {
     pub default: Option<DefaultValue>,
     pub default_with: Option<LitStr>,
     pub validator: Option<Validators>,
+    #[darling(default)]
+    pub process_with: Option<String>,
     pub key: bool, // for entity
     pub visible: Option<Visible>,
     pub secret: bool,
@@ -222,6 +230,7 @@ pub struct Argument {
 pub struct Object {
     pub internal: bool,
     pub name: Option<String>,
+    pub name_type: bool,
     pub rename_fields: Option<RenameRule>,
     pub rename_args: Option<RenameRule>,
     pub cache_control: CacheControl,
@@ -275,7 +284,6 @@ pub struct ObjectField {
     #[darling(default, multiple)]
     pub derived: Vec<DerivedField>,
     pub flatten: bool,
-    pub oneof: bool,
 }
 
 #[derive(FromMeta, Default, Clone)]
@@ -372,6 +380,8 @@ pub struct InputObjectField {
     pub skip: bool,
     #[darling(default)]
     pub skip_input: bool,
+    #[darling(default)]
+    pub process_with: Option<String>,
     // for SimpleObject
     #[darling(default)]
     pub skip_output: bool,
@@ -481,8 +491,6 @@ pub struct InterfaceField {
     pub requires: Option<String>,
     #[darling(default)]
     pub visible: Option<Visible>,
-    #[darling(default)]
-    pub oneof: bool,
 }
 
 #[derive(FromVariant)]
@@ -547,6 +555,8 @@ pub struct SubscriptionFieldArgument {
     pub default: Option<DefaultValue>,
     pub default_with: Option<LitStr>,
     pub validator: Option<Validators>,
+    #[darling(default)]
+    pub process_with: Option<String>,
     pub visible: Option<Visible>,
     pub secret: bool,
 }
@@ -560,7 +570,6 @@ pub struct SubscriptionField {
     pub guard: Option<SpannedValue<String>>,
     pub visible: Option<Visible>,
     pub complexity: Option<ComplexityType>,
-    pub oneof: bool,
 }
 
 #[derive(FromField)]
@@ -761,7 +770,6 @@ pub struct ComplexObjectField {
     #[darling(multiple)]
     pub derived: Vec<DerivedField>,
     pub flatten: bool,
-    pub oneof: bool,
 }
 
 #[derive(FromMeta, Default)]
