@@ -1,20 +1,21 @@
+use std::{iter::FromIterator, str::FromStr};
+
 use proc_macro::TokenStream;
 use proc_macro2::Ident;
 use quote::quote;
-use std::iter::FromIterator;
-use std::str::FromStr;
-use syn::ext::IdentExt;
 use syn::{
-    punctuated::Punctuated, Block, Error, FnArg, ImplItem, ItemImpl, Pat, ReturnType, Token, Type,
-    TypeReference,
+    ext::IdentExt, punctuated::Punctuated, Block, Error, FnArg, ImplItem, ItemImpl, Pat,
+    ReturnType, Token, Type, TypeReference,
 };
 
-use crate::args::{self, ComplexityType, RenameRuleExt, RenameTarget};
-use crate::output_type::OutputType;
-use crate::utils::{
-    extract_input_args, gen_deprecation, generate_default, generate_guards, get_cfg_attrs,
-    get_crate_name, get_rustdoc, get_type_path_and_name, parse_complexity_expr,
-    parse_graphql_attrs, remove_graphql_attrs, visible_fn, GeneratorResult,
+use crate::{
+    args::{self, ComplexityType, RenameRuleExt, RenameTarget},
+    output_type::OutputType,
+    utils::{
+        extract_input_args, gen_deprecation, generate_default, generate_guards, get_cfg_attrs,
+        get_crate_name, get_rustdoc, get_type_path_and_name, parse_complexity_expr,
+        parse_graphql_attrs, remove_graphql_attrs, visible_fn, GeneratorResult,
+    },
 };
 
 pub fn generate(
@@ -25,10 +26,18 @@ pub fn generate(
     let (self_ty, self_name) = get_type_path_and_name(item_impl.self_ty.as_ref())?;
     let (impl_generics, _, where_clause) = item_impl.generics.split_for_impl();
     let extends = object_args.extends;
-    let gql_typename = object_args
-        .name
-        .clone()
-        .unwrap_or_else(|| RenameTarget::Type.rename(self_name.clone()));
+    let gql_typename = if !object_args.name_type {
+        object_args
+            .name
+            .as_ref()
+            .map(|name| quote!(::std::borrow::Cow::Borrowed(#name)))
+            .unwrap_or_else(|| {
+                let name = RenameTarget::Type.rename(self_name.clone());
+                quote!(::std::borrow::Cow::Borrowed(#name))
+            })
+    } else {
+        quote!(<Self as #crate_name::TypeName>::type_name())
+    };
 
     let desc = if object_args.use_type_description {
         quote! { ::std::option::Option::Some(<Self as #crate_name::Description>::description()) }
@@ -141,14 +150,6 @@ pub fn generate(
                     return Err(Error::new_spanned(&method, "Must be asynchronous").into());
                 }
 
-                if method_args.oneof {
-                    return Err(Error::new_spanned(
-                        &method,
-                        "The `entity` and `oneof` attributes cannot be specified at the same time.",
-                    )
-                    .into());
-                }
-
                 let args = extract_input_args::<args::Argument>(&crate_name, method)?;
 
                 let ty = match &method.sig.output {
@@ -189,7 +190,7 @@ pub fn generate(
                     if is_key {
                         get_federation_key.push(quote! {
                             if let Some(fields) = <#ty as #crate_name::InputType>::federation_fields() {
-                                key_str.push(format!("{} {}", #name, fields));                                
+                                key_str.push(format!("{} {}", #name, fields));
                             } else {
                                 key_str.push(#name.to_string());
                             }
@@ -286,7 +287,6 @@ pub fn generate(
                     let ident = &method.sig.ident;
 
                     schema_fields.push(quote! {
-                        #crate_name::static_assertions::assert_impl_one!(#ty: #crate_name::ObjectType);
                         <#ty>::create_type_info(registry);
                         if let #crate_name::registry::MetaType::Object { fields: obj_fields, .. } =
                             registry.create_fake_output_type::<#ty>() {
@@ -334,86 +334,50 @@ pub fn generate(
                     }
                 };
 
-                let mut args = extract_input_args::<args::Argument>(&crate_name, method)?;
+                let args = extract_input_args::<args::Argument>(&crate_name, method)?;
                 let mut schema_args = Vec::new();
                 let mut use_params = Vec::new();
                 let mut get_params = Vec::new();
-                let mut is_oneof_field = false;
 
-                if method_args.oneof {
-                    is_oneof_field = true;
-
-                    if args.len() != 1 {
-                        return Err(Error::new_spanned(
-                            &method,
-                            "The `oneof` field requires exactly one argument.",
-                        )
-                        .into());
-                    }
-                    let (ident, ty, argument) = args.pop().unwrap();
+                for (
+                    ident,
+                    ty,
+                    args::Argument {
+                        name,
+                        desc,
+                        default,
+                        default_with,
+                        process_with,
+                        validator,
+                        visible,
+                        secret,
+                        ..
+                    },
+                ) in &args
+                {
+                    let name = name.clone().unwrap_or_else(|| {
+                        object_args
+                            .rename_args
+                            .rename(ident.ident.unraw().to_string(), RenameTarget::Argument)
+                    });
+                    let desc = desc
+                        .as_ref()
+                        .map(|s| quote! {::std::option::Option::Some(#s)})
+                        .unwrap_or_else(|| quote! {::std::option::Option::None});
+                    let default = generate_default(default, default_with)?;
+                    let schema_default = default
+                        .as_ref()
+                        .map(|value| {
+                            quote! {
+                                ::std::option::Option::Some(::std::string::ToString::to_string(
+                                    &<#ty as #crate_name::InputType>::to_value(&#value)
+                                ))
+                            }
+                        })
+                        .unwrap_or_else(|| quote! {::std::option::Option::None});
 
+                    let visible = visible_fn(visible);
                     schema_args.push(quote! {
-                        #crate_name::static_assertions::assert_impl_one!(#ty: #crate_name::OneofObjectType);
-                        if let #crate_name::registry::MetaType::InputObject { input_fields, .. } = registry.create_fake_input_type::<#ty>() {
-                            args.extend(input_fields);
-                        }
-                    });
-                    use_params.push(quote! { #ident });
-
-                    let validators = argument
-                        .validator
-                        .clone()
-                        .unwrap_or_default()
-                        .create_validators(
-                            &crate_name,
-                            quote!(&#ident),
-                            quote!(#ty),
-                            Some(quote!(.map_err(|err| err.into_server_error(__pos)))),
-                        )?;
-                    get_params.push(quote! {
-                        #[allow(non_snake_case, unused_variables)]
-                        let (__pos, #ident) = ctx.oneof_param_value::<#ty>()?;
-                        #validators
-                    });
-                } else {
-                    for (
-                        ident,
-                        ty,
-                        args::Argument {
-                            name,
-                            desc,
-                            default,
-                            default_with,
-                            validator,
-                            visible,
-                            secret,
-                            ..
-                        },
-                    ) in &args
-                    {
-                        let name = name.clone().unwrap_or_else(|| {
-                            object_args
-                                .rename_args
-                                .rename(ident.ident.unraw().to_string(), RenameTarget::Argument)
-                        });
-                        let desc = desc
-                            .as_ref()
-                            .map(|s| quote! {::std::option::Option::Some(#s)})
-                            .unwrap_or_else(|| quote! {::std::option::Option::None});
-                        let default = generate_default(default, default_with)?;
-                        let schema_default = default
-                            .as_ref()
-                            .map(|value| {
-                                quote! {
-                                    ::std::option::Option::Some(::std::string::ToString::to_string(
-                                        &<#ty as #crate_name::InputType>::to_value(&#value)
-                                    ))
-                                }
-                            })
-                            .unwrap_or_else(|| quote! {::std::option::Option::None});
-
-                        let visible = visible_fn(visible);
-                        schema_args.push(quote! {
                             args.insert(::std::borrow::ToOwned::to_owned(#name), #crate_name::registry::MetaInputValue {
                                 name: #name,
                                 description: #desc,
@@ -424,29 +388,43 @@ pub fn generate(
                             });
                         });
 
-                        let param_ident = &ident.ident;
-                        use_params.push(quote! { #param_ident });
+                    let param_ident = &ident.ident;
+                    use_params.push(quote! { #param_ident });
 
-                        let default = match default {
-                            Some(default) => {
-                                quote! { ::std::option::Option::Some(|| -> #ty { #default }) }
+                    let default = match default {
+                        Some(default) => {
+                            quote! { ::std::option::Option::Some(|| -> #ty { #default }) }
+                        }
+                        None => quote! { ::std::option::Option::None },
+                    };
+
+                    let process_with = match process_with.as_ref() {
+                        Some(fn_path) => {
+                            let fn_path: syn::ExprPath = syn::parse_str(fn_path)?;
+                            quote! {
+                                #fn_path(&mut #param_ident);
                             }
-                            None => quote! { ::std::option::Option::None },
-                        };
+                        }
+                        None => Default::default(),
+                    };
 
-                        let validators = validator.clone().unwrap_or_default().create_validators(
-                            &crate_name,
-                            quote!(&#ident),
-                            quote!(#ty),
-                            Some(quote!(.map_err(|err| err.into_server_error(__pos)))),
-                        )?;
+                    let validators = validator.clone().unwrap_or_default().create_validators(
+                        &crate_name,
+                        quote!(&#ident),
+                        quote!(#ty),
+                        Some(quote!(.map_err(|err| err.into_server_error(__pos)))),
+                    )?;
 
-                        get_params.push(quote! {
-                            #[allow(non_snake_case, unused_variables)]
-                            let (__pos, #ident) = ctx.param_value::<#ty>(#name, #default)?;
-                            #validators
-                        });
-                    }
+                    let mut non_mut_ident = ident.clone();
+                    non_mut_ident.mutability = None;
+                    get_params.push(quote! {
+                        #[allow(non_snake_case, unused_variables, unused_mut)]
+                        let (__pos, mut #non_mut_ident) = ctx.param_value::<#ty>(#name, #default)?;
+                        #process_with
+                        #validators
+                        #[allow(non_snake_case, unused_variables)]
+                        let #ident = #non_mut_ident;
+                    });
                 }
 
                 let ty = match &method.sig.output {
@@ -531,7 +509,6 @@ pub fn generate(
                         requires: #requires,
                         visible: #visible,
                         compute_complexity: #complexity,
-                        oneof: #is_oneof_field,
                     });
                 });
 
@@ -649,12 +626,12 @@ pub fn generate(
             #[#crate_name::async_trait::async_trait]
             impl #impl_generics #crate_name::OutputType for #self_ty #where_clause {
                 fn type_name() -> ::std::borrow::Cow<'static, ::std::primitive::str> {
-                    ::std::borrow::Cow::Borrowed(#gql_typename)
+                    #gql_typename
                 }
 
                 fn create_type_info(registry: &mut #crate_name::registry::Registry) -> ::std::string::String {
-                    let ty = registry.create_output_type::<Self, _>(|registry| #crate_name::registry::MetaType::Object {
-                        name: ::std::borrow::ToOwned::to_owned(#gql_typename),
+                    let ty = registry.create_output_type::<Self, _>(#crate_name::registry::MetaTypeId::Object, |registry| #crate_name::registry::MetaType::Object {
+                        name: ::std::borrow::Cow::into_owned(#gql_typename),
                         description: #desc,
                         fields: {
                             let mut fields = #crate_name::indexmap::IndexMap::new();
@@ -692,7 +669,7 @@ pub fn generate(
 
             impl #impl_generics #self_ty #where_clause {
                 fn __internal_create_type_info(registry: &mut #crate_name::registry::Registry, name: &str) -> ::std::string::String  where Self: #crate_name::OutputType {
-                    let ty = registry.create_output_type::<Self, _>(|registry| #crate_name::registry::MetaType::Object {
+                    let ty = registry.create_output_type::<Self, _>(#crate_name::registry::MetaTypeId::Object, |registry| #crate_name::registry::MetaType::Object {
                         name: ::std::borrow::ToOwned::to_owned(name),
                         description: #desc,
                         fields: {
